@@ -263,11 +263,7 @@ QString XdgDirs::configHome(bool createDir)
 QStringList XdgDirs::dataDirs(const QString &postfix)
 {
     QString d = QFile::decodeName(qgetenv("XDG_DATA_DIRS"));
-#if (QT_VERSION >= QT_VERSION_CHECK(5,15,0))
     QStringList dirs = d.split(QLatin1Char(':'), Qt::SkipEmptyParts);
-#else
-    QStringList dirs = d.split(QLatin1Char(':'), QString::SkipEmptyParts);
-#endif
 
     if (dirs.isEmpty()) {
         dirs.append(QString::fromLatin1("/usr/local/share"));
@@ -295,11 +291,7 @@ QStringList XdgDirs::configDirs(const QString &postfix)
     if (env.isEmpty())
         dirs.append(QString::fromLatin1("/etc/xdg"));
     else
-#if (QT_VERSION >= QT_VERSION_CHECK(5,15,0))
         dirs = env.split(QLatin1Char(':'), Qt::SkipEmptyParts);
-#else
-        dirs = env.split(QLatin1Char(':'), QString::SkipEmptyParts);
-#endif
 
     cleanAndAddPostfix(dirs, postfix);
     return dirs;
