@@ -227,7 +227,7 @@ void XdgMenuPrivate::load(const QString& fileName)
         qWarning() << QString::fromLatin1("%1 not loading: %2").arg(fileName, file.errorString());
         return;
     }
-    mXml.setContent(&file, true);
+    mXml.setContent(&file, QDomDocument::ParseOption::UseNamespaceProcessing);
 }
 
 
@@ -666,6 +666,7 @@ QString XdgMenu::getMenuFileName(const QString& baseName)
     // rest files ordered by priority (descending)
     wellKnownFiles << QLatin1String("kde4-applications.menu");
     wellKnownFiles << QLatin1String("kde-applications.menu");
+    wellKnownFiles << QLatin1String("plasma-applications.menu");
     wellKnownFiles << QLatin1String("gnome-applications.menu");
     wellKnownFiles << QLatin1String("lxde-applications.menu");
 
